﻿<?xml version="1.0" encoding="UTF-8"?>
<!--
'SchemaDB' database query system in XSLT for Weblide, http://alexandre.alapetite.fr/distribution/weblide/

- Limitations: .NET 2.0 <= XML 1.0, XSLT 1.0, XPath 2.0, XQuery 1.0

	==Entry example (given by Weblide)==
<svg
 xmlns:sdb2="http://alexandre.alapetite.fr/distribution/weblide/#v2"
 xmlns="http://www.w3.org/2000/svg"
 version="1.1"
 baseProfile="basic"
 viewBox="0 0 100 100">
	<doctype	/*Always in the default namespace*/
	 public="-//W3C//DTD SVG 1.1 Basic//EN"	/*Name of the DTD, if any*/
	 allowNonOfficial="false"
	 allowBeta="false"
	 useAccessibility="true"
	 useAdvises="false"/>
	<sdb2:ns name="http://www.w3.org/2000/svg"/>	/*One ns element per namespace*/
</svg>

	==Output example (returned to Weblide)==
<dtd xmlns="http://alexandre.alapetite.fr/distribution/weblide/#v2">
	<namespace name="http://www.w3.org/2000/svg">
		<schema type="dtd" href="http://www.w3.org/Graphics/SVG/1.1/DTD/svg11-basic.dtd" rel="normative" official="official" active="true"/>
	</namespace>
	<namespace name="http://www.w3.org/2000/svg">
		<schema type="relaxng" href="http://www.w3.org/Graphics/SVG/1.1/rng/svg11-tiny.rng" rel="normative" official="official" active="true"/>
	</namespace>
</dtd>

Version: 2006-06-11
________________________________________________________________________
Copyright 2006-2008, Alexandre Alapetite [http://alexandre.alapetite.fr/cv/]

 CeCILL-C Free software license (CEA/CNRS/INRA, France)
  [http://www.cecill.info/licences/Licence_CeCILL-C_V1-en.html]
  Compatible GNU/LGPL

- Attribution. You must give the original author credit
  [http://alexandre.alapetite.fr/cv/].
- Share Alike. If you alter, transform or extend this library,
  you may distribute the resulting work only under one of the following licences:
  CeCILL-C, CeCILL, GNU LGPL or GPL, Creative Commons BY-SA or BY-NC-SA, Mozilla Public, Apache.
- Any of these conditions can be waived if you get permission from Alexandre Alapetite.
- Read the licence for details regarding limited warranty and liability.
- Please send to the original author the modifications you make,
  in order to improve this library for the benefit of everybody.
- If you want to distribute this code, when possible please provide a link to
  its original location [http://alexandre.alapetite.fr/distribution/weblide/].
-->
<xsl:stylesheet
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns="http://alexandre.alapetite.fr/distribution/weblide/#v2"
 xmlns:sdb="http://alexandre.alapetite.fr/distribution/weblide/#v2"
 version="1.0"
 xml:lang="en-GB">

<xsl:output
 method="xml"
 indent="no"
 omit-xml-declaration="no"
 encoding="UTF-8"/>

<xsl:template match="/*">
	<xsl:variable name="sdb_root" select="."/>
	<xsl:variable name="sdb_dtd" select="*[local-name()='doctype']/@*[local-name()='public']"/>
	<dtd><!-- TODO: Optimise -->
		<xsl:variable name="sdb_languages" select="document('SchemaDB.xml')/sdb:SchemaDB/sdb:languages[@active='true']"/>
		<xsl:for-each select="sdb:ns">
			<xsl:variable name="sdb_language" select="$sdb_languages/sdb:language[@namespace=current()/@name]"/>
			<xsl:variable name="sdb_0" select="$sdb_language/sdb:root[@name=local-name($sdb_root) and @namespace=namespace-uri($sdb_root)]/.."/>
			<xsl:variable name="sdb_1" select="$sdb_0/sdb:dtd/sdb:public[@name=$sdb_dtd]/../sdb:schema"/>
			<xsl:variable name="sdb_2" select="$sdb_0/sdb:dtd[@default='default']/sdb:schema"/>
			<xsl:variable name="sdb_first">
				<xsl:choose>
					<xsl:when test="$sdb_1">
						<xsl:choose>
							<xsl:when test="$sdb_1/../sdb:profile">
								<xsl:apply-templates select="$sdb_1/../sdb:profile">
									<xsl:with-param name="sdb_root" select="$sdb_root"/>
								</xsl:apply-templates>
							</xsl:when>
							<xsl:otherwise>
								<xsl:apply-templates select="$sdb_1">
									<xsl:with-param name="sdb_doctype" select="$sdb_root/*[local-name()='doctype']"/>
								</xsl:apply-templates>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:when test="$sdb_2">
						<xsl:choose>
							<xsl:when test="$sdb_2/../sdb:profile">
								<xsl:apply-templates select="$sdb_2/../sdb:profile">
									<xsl:with-param name="sdb_root" select="$sdb_root"/>
								</xsl:apply-templates>
							</xsl:when>
							<xsl:otherwise>
								<xsl:apply-templates select="$sdb_2">
									<xsl:with-param name="sdb_doctype" select="$sdb_root/*[local-name()='doctype']"/>
								</xsl:apply-templates>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
				</xsl:choose>
			</xsl:variable>
			<xsl:copy-of select="$sdb_first"/><!-- TODO: Optimise -->
			<xsl:variable name="sdb_3" select="$sdb_language[not(contains($sdb_first,@namespace))]/sdb:roots[@default='default']/../sdb:dtd[@default='default']/sdb:schema[@active='true']"/>
			<xsl:if test="$sdb_3">
				<xsl:choose>
					<xsl:when test="$sdb_3/../sdb:profile">
						<xsl:apply-templates select="$sdb_3/../sdb:profile">
							<xsl:with-param name="sdb_root" select="$sdb_root"/>
						</xsl:apply-templates>
					</xsl:when>
					<xsl:otherwise>
						<xsl:apply-templates select="$sdb_3">
							<xsl:with-param name="sdb_doctype" select="$sdb_root/*[local-name()='doctype']"/>
						</xsl:apply-templates>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:if>
		</xsl:for-each>
	</dtd>
</xsl:template>

<xsl:template match="/sdb:SchemaDB/sdb:languages/sdb:language/sdb:dtd/sdb:profile">
	<xsl:param name="sdb_root"/>
	<xsl:variable name="sdb_schemaOk">
		<xsl:for-each select="sdb:constraint">
			<xsl:if test="not(current()/@default and not($sdb_root/@*[local-name()=current()/@name])) and not($sdb_root/@*[local-name()=current()/@name]=current()/@value)">
				rejected<!-- TODO: Take into account the namespace -->
			</xsl:if>
		</xsl:for-each>
	</xsl:variable>
	<xsl:if test="not(contains($sdb_schemaOk,'rejected'))">
		<xsl:apply-templates select="../sdb:schema">
			<xsl:with-param name="sdb_doctype" select="$sdb_root/*[local-name()='doctype']"/>
		</xsl:apply-templates>
	</xsl:if>
</xsl:template>

<xsl:template match="/sdb:SchemaDB/sdb:languages/sdb:language/sdb:dtd/sdb:schema">
	<xsl:param name="sdb_doctype"/>
	<xsl:value-of select="../../@namespace"/>
	<xsl:if test="@active='true' and (@official or $sdb_doctype/@allowNonOfficial='true') and (not(@beta) or $sdb_doctype/@allowBeta='true') and (not(@rel='accessibility') or $sdb_doctype/@useAccessibility='true') and (not(@rel='advises') or $sdb_doctype/@useAdvises='true')">
		<namespace>
			<xsl:attribute name="name"><xsl:value-of select="../../@namespace"/></xsl:attribute>
			<xsl:copy-of select="."/>
		</namespace>
	</xsl:if>
</xsl:template>

</xsl:stylesheet>
