<?xml version="1.0" encoding="UTF-8"?>
<!--
	XSLT to transform some RDF Schema rules in XML Schema.
	
	By Alexandre Alapetite [http://alexandre.alapetite.fr/cv/]
		for Weblide [http://alexandre.alapetite.fr/distribution/weblide/]
		on 2006-05-21 - 2006-05-22. Alpha version.
	
	Contains code from Dimitre Novatchev, 2001 [http://sources.redhat.com/ml/xsl-list/2001-08/msg00243.html]
	
	Copyright 2006, Licence: Creative Commons "Attribution-ShareAlike 2.0 France" BY-SA (FR),
	http://creativecommons.org/licenses/by-sa/2.0/fr/
	http://alexandre.alapetite.fr/divers/apropos/#by-sa
	- Attribution. You must give the original author credit
	- Share Alike. If you alter, transform, or build upon this work,
	  you may distribute the resulting work only under a license identical to this one
	  (Can be included in GPL/LGPL projects)
	- The French law is authoritative
	- Any of these conditions can be waived if you get permission from Alexandre Alapetite
	- Please send to Alexandre Alapetite the modifications you make,
	  in order to improve this file for the benefit of everybody
-->
<xsl:stylesheet
 version="1.0"
 xmlns="http://www.w3.org/2001/XMLSchema"
 xmlns:xs="http://www.w3.org/2001/XMLSchema"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
 xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
 xmlns:owl="http://www.w3.org/2002/07/owl#"
 xmlns:dc="http://purl.org/dc/elements/1.1/">

<xsl:output
 method="xml"
 version="1.0"
 encoding="UTF-8"
 omit-xml-declaration="no"
 indent="yes"
 media-type="application/xml"/>

<xsl:template match="/">
	<xsl:apply-templates select="//rdf:RDF"/>
</xsl:template>

<xsl:template name="reverse">
	<!-- Reverse string by Dimitre Novatchev, 2001 [http://sources.redhat.com/ml/xsl-list/2001-08/msg00243.html] -->
	<xsl:param name="theString"/>
	<xsl:variable name="thisLength" select="string-length($theString)"/>
	<xsl:choose>
		<xsl:when test="$thisLength = 1">
			<xsl:value-of select="$theString"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:variable name="restReverse">
				<xsl:call-template name="reverse">
					<xsl:with-param name="theString" select="substring($theString, 1, $thisLength -1)"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:value-of select="concat(substring($theString,$thisLength,1),$restReverse)"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="//rdf:RDF">
	<xsl:variable name="ontology"><!-- Search the targeted namespace -->
		<xsl:choose>
			<xsl:when test="contains(rdf:Property[1]/@rdf:about,'#')">
				<xsl:value-of select="concat(substring-before(rdf:Property[1]/@rdf:about,'#'),'#')"/>
			</xsl:when>
			<xsl:when test="owl:Ontology/@rdf:about and (string-length(owl:Ontology/@rdf:about) &gt; 0)">
				<xsl:value-of select="owl:Ontology/@rdf:about"/>
			</xsl:when>
			<xsl:when test="rdfs:Class[1]/rdfs:isDefinedBy/@rdf:resource">
				<xsl:value-of select="rdfs:Class[1]/rdfs:isDefinedBy/@rdf:resource"/>
			</xsl:when>
			<xsl:when test="contains(rdfs:Class[1]/@rdf:about,'#')">
				<xsl:value-of select="concat(substring-before(rdfs:Class[1]/@rdf:about,'#'),'#')"/>
			</xsl:when>
			<xsl:when test="contains(rdfs:Class[1]/@rdf:about,'/')">
				<xsl:variable name="expandedName1">
					<xsl:call-template name="reverse">
						<xsl:with-param name="theString" select="rdfs:Class[1]/@rdf:about"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:call-template name="reverse">
					<xsl:with-param name="theString" select="concat('/',substring-after($expandedName1,'/'))"/>
				</xsl:call-template>
			</xsl:when>
			<!--<xsl:otherwise>
				<xsl:value-of select="namespace-uri(/)"/>
			</xsl:otherwise>-->
		</xsl:choose>
	</xsl:variable>
	<xs:schema
	 elementFormDefault="qualified"
	 xml:lang="en-GB">
		<xsl:attribute name="targetNamespace">
			<xsl:value-of select="$ontology"/>
		</xsl:attribute>
		<xs:annotation>
			<xs:documentation>
				XML Schema for "<xsl:value-of select="owl:Ontology/@dc:title"/>".
				Generated automatically with rdf2xsd.xsl
					from Alexandre Alapetite [http://alexandre.alapetite.fr/cv/]
					for Weblide [http://alexandre.alapetite.fr/distribution/weblide/]
					Copyright 2006, Licence: Creative Commons "Attribution-ShareAlike 2.0 France" BY-SA (FR) [http://creativecommons.org/licenses/by-sa/2.0/fr/]
			</xs:documentation>
		</xs:annotation>
		<xsl:apply-templates mode="name" select="rdf:Property[starts-with(@rdf:about,$ontology)]">
			<xsl:with-param name="ontology" select="$ontology"/>
		</xsl:apply-templates>
		<xsl:apply-templates mode="name" select="rdfs:Class[starts-with(@rdf:about,$ontology)]">
			<xsl:with-param name="ontology" select="$ontology"/>
		</xsl:apply-templates>
	</xs:schema>
</xsl:template>

<xsl:template match="//rdf:RDF/rdfs:Class" mode="name">
	<xsl:param name="ontology"/>
	<!--<xs:group>
		<xsl:attribute name="name">
			<xsl:value-of select="concat('_',substring-after(@rdf:about,$ontology),'_Group')"/>
		</xsl:attribute>
		<xs:choice>
			<xsl:apply-templates mode="ref" select="../rdf:Property/rdfs:domain[@rdf:resource=current()/@rdf:about]/..">
				<xsl:with-param name="ontology" select="$ontology"/>
			</xsl:apply-templates>
			<xsl:apply-templates mode="ref" select="../rdfs:Class[@rdf:about=current()/rdfs:subClassOf/@rdf:resource]">
				<xsl:with-param name="ontology" select="$ontology"/>
			</xsl:apply-templates>
		</xs:choice>
	</xs:group>
	<xs:complexType>
		<xsl:attribute name="name">
			<xsl:value-of select="concat('_',substring-after(@rdf:about,$ontology),'_Type')"/>
		</xsl:attribute>
		<xs:choice minOccurs="0" maxOccurs="unbounded">
			<xs:group>
				<xsl:attribute name="ref">
					<xsl:value-of select="concat('_',substring-after(@rdf:about,$ontology),'_Group')"/>
				</xsl:attribute>
			</xs:group>
		</xs:choice>
	</xs:complexType>-->
	<xs:element type="xs:anyType">
		<xsl:attribute name="name">
			<xsl:value-of select="substring-after(@rdf:about,$ontology)"/>
		</xsl:attribute>
		<!--<xsl:attribute name="type">
			<xsl:value-of select="concat('_',substring-after(@rdf:about,$ontology),'_Type')"/>
		</xsl:attribute>-->
	</xs:element>
</xsl:template>

<!--<xsl:template match="//rdf:RDF/rdfs:Class" mode="ref">
	<xsl:param name="ontology"/>
	<xs:group>
		<xsl:attribute name="ref">
			<xsl:value-of select="concat('_',substring-after(@rdf:about,$ontology),'_Group')"/>
		</xsl:attribute>
	</xs:group>
</xsl:template>-->

<xsl:template match="//rdf:RDF/rdf:Property" mode="name">
	<xsl:param name="ontology"/>
	<xs:element type="xs:anyType">
		<xsl:attribute name="name">
			<xsl:value-of select="substring-after(@rdf:about,$ontology)"/>
		</xsl:attribute>
	</xs:element>
</xsl:template>

<!--<xsl:template match="//rdf:RDF/rdf:Property" mode="ref">
	<xsl:param name="ontology"/>
	<xs:element>
		<xsl:attribute name="ref">
			<xsl:value-of select="substring-after(@rdf:about,$ontology)"/>
		</xsl:attribute>
	</xs:element>
</xsl:template>-->

</xsl:stylesheet>
